/*
Copyright 2021 Alibaba Group Holding Limited.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package client

import (
	"context"

	ctrl "sigs.k8s.io/controller-runtime"
	"sigs.k8s.io/controller-runtime/pkg/client"
)

type bypassCacheClient struct {
	client.Client

	apiReader client.Reader
}

func (b *bypassCacheClient) Get(ctx context.Context, key client.ObjectKey, obj client.Object) error {
	return b.apiReader.Get(ctx, key, obj)
}

func (b *bypassCacheClient) List(ctx context.Context, list client.ObjectList, opts ...client.ListOption) error {
	return b.apiReader.List(ctx, list, opts...)
}

func NewClientBypassCache(mgr ctrl.Manager) client.Client {
	return &bypassCacheClient{
		Client:    mgr.GetClient(),
		apiReader: mgr.GetAPIReader(),
	}
}
